#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test TransformationPlugin module."""

from __future__ import print_function
from __future__ import absolute_import
import sys
import unittest
from mock import patch, MagicMock as Mock

from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith_equals, assertListContentEquals, assertInImproved
from six.moves import range

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin'

# pylint: disable=protected-access


class TransformationPluginTestCase(unittest.TestCase):
  """Base class for the TransformationPlugin test cases."""

  def setUp(self):
    """set up the objects."""
    mocked_modules = {'DIRAC.DataManagementSystem.Client.DataManager': Mock(),
                       'DIRAC.Resources.Catalog.FileCatalog.FileCatalog': Mock(),
                       'DIRAC.Resources.Catalog.FileCatalog': Mock()}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    self.tfp = None  # Stores the TransformationPlugin object

  def tearDown(self):
    self.module_patcher.stop()

  def test_limited_add_up_to_maximum(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK
    dataman_mock = Mock()
    util_mock = Mock()
    util_mock.groupByReplicas.return_value = S_OK(
        [('testSE', ['mylfn1', 'testLFN2']), ('testSE', ['other_test_lfn']),
         ('testSE', ['mylfn']), ('testSE', ['MyLFN'])])
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK(
        [({'Status': 'Processed'}, 3), ({'Status': 'junk'}, 6), ({'Status': 'Ignore_me'}, 8),
         ({'Status': 'Assigned'}, 1)])
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('Limited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 6
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    test_data = [
        ('/my/file/here_abc', ['myTestSE']),
        ('/my/file/other_file.txt', ['secretSE', 'StorageSE', 'TestSE']),
        ('/my/file/newfile.pdf', ['TestSE']),
        ('/my/file/a', []), ('/dir/somefile', [''])]
    self.tfp.setInputData(test_data)
    assertDiracSucceedsWith_equals(self.tfp.run(), [
        ('', ['mylfn1', 'testLFN2']), ('', ['other_test_lfn'])], self)
    util_mock.groupByReplicas.assert_called_once_with(test_data, 'Processed')
    trans_mock.getCounters.assert_called_once_with('TransformationFiles', ['Status'],
                                                    {'TransformationID': 78456})

  def test_limited_add_all(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK
    dataman_mock = Mock()
    new_tasks = [('otherSE', ['unique_lfn', 'testme_lfn']), ('testSE', ['LFN_to_add']),
                  ('testSE', ['LFN:/mydir/subdir/file.stdhep']), ('testSE', ['testSE/storage/LFN.txt'])]
    util_mock = Mock()
    util_mock.groupByReplicas.return_value = S_OK(new_tasks)
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK(
        [({'Status': 'Processed'}, 3), ({'Status': 'junk'}, 6), ({'Status': 'Ignore_me'}, 8),
         ({'Status': 'Assigned'}, 1)])
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('Limited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 12
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    self.tfp.setInputData([
        ('/my/file/here_abc', ['myTestSE']),
        ('/my/file/other_file.txt', ['secretSE', 'StorageSE', 'TestSE']),
        ('/my/file/newfile.pdf', ['TestSE']),
        ('/my/file/a', []), ('/dir/somefile', [''])])
    assertDiracSucceedsWith_equals(self.tfp.run(), [('', tup) for (_se, tup) in new_tasks], self)

  def test_limited_too_many(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK
    dataman_mock = Mock()
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK([({'Status': 'Processed'}, 2)] * 23
                                                + [({'Status': 'Assigned'}, 1)] * 15
                                                + [({'Status': 'junk'}, 0)] * 4)
    self.tfp = TransformationPlugin('Limited', trans_mock, dataman_mock)
    self.tfp.params['MaxNumberOfTasks'] = 59
    self.tfp.params['TransformationID'] = 78456
    assertDiracSucceedsWith_equals(self.tfp.run(), [], self)

  def test_limited_getcounters_fails(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_ERROR
    dataman_mock = Mock()
    util_mock = Mock()
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_ERROR('my_test_getcount_err')
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('Limited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 6
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    assertDiracFailsWith(self.tfp.run(), 'my_test_getcount_err', self)

  def test_limited_groupbyreplicas_fails(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK, S_ERROR
    dataman_mock = Mock()
    util_mock = Mock()
    util_mock.groupByReplicas.return_value = S_ERROR('group_test_err')
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK(
        [({'Status': 'Processed'}, 3), ({'Status': 'junk'}, 6), ({'Status': 'Ignore_me'}, 8),
         ({'Status': 'Assigned'}, 1)])
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('Limited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 12
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    self.tfp.setInputData([
        ('/my/file/here_abc', ['myTestSE']),
        ('/my/file/other_file.txt', ['secretSE', 'StorageSE', 'TestSE']),
        ('/my/file/newfile.pdf', ['TestSE']),
        ('/my/file/a', []), ('/dir/somefile', [''])])
    assertDiracFailsWith(self.tfp.run(), 'group_test_err', self)

  def test_sliced(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    dataman_mock = Mock()
    trans_mock = Mock()
    self.tfp = TransformationPlugin('Sliced', dataman_mock, trans_mock)
    self.tfp.setInputData({
        '/my/file/here_abc': ['myTestSE'],
        '/my/file/other_file.txt': ['secretSE', 'StorageSE', 'TestSE'],
        '/my/file/newfile.pdf': ['TestSE'],
        '/my/file/a': [], '/dir/somefile': ['']})
    result = self.tfp.run()
    assertDiracSucceeds(result, self)
    assertListContentEquals(result['Value'], [
        ('', ['/my/file/here_abc']), ('', ['/my/file/other_file.txt']),
        ('', ['/my/file/newfile.pdf']), ('', ['/my/file/a']), ('', ['/dir/somefile'])], self)

  def test_sliced_empty(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    dataman_mock = Mock()
    trans_mock = Mock()
    self.tfp = TransformationPlugin('Sliced', dataman_mock, trans_mock)
    self.tfp.setInputData({})
    assertDiracSucceedsWith_equals(self.tfp.run(), [], self)

  def test_sliced_limited_add_up_to_maximum(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK
    dataman_mock = Mock()
    util_mock = Mock()
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK(
        [({'Status': 'Processed'}, 4), ({'Status': 'junk'}, 6), ({'Status': 'Ignore_me'}, 8),
         ({'Status': 'Assigned'}, 6)])
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('SlicedLimited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 12
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    self.tfp.setInputData({
        '/my/file/here_abc': ['myTestSE'],
        '/my/file/other_file.txt': ['secretSE', 'StorageSE', 'TestSE'],
        '/my/file/newfile.pdf': ['TestSE'],
        '/my/file/a': [], '/dir/somefile': ['']})
    result = self.tfp.run()
    assertDiracSucceeds(result, self)
    to_check = result['Value']
    print(result)
    assertEqualsImproved(
        (len(to_check), to_check[0][0], to_check[1][0], len(to_check[0][1]), len(to_check[1][1])),
        (2, '', '', 1, 1), self)  # Checks that two tasks are added, that the two tasks have the form ('', [ a ])
    self.assertNotEqual(to_check[0][1][0], to_check[1][1][0])  # Checks that the same lfn isnt added twice
    # Checks that the LFN is one of those expected (since keys() iteration is random for dictionaries)
    expected = ['/my/file/here_abc', '/my/file/other_file.txt', '/my/file/newfile.pdf',
                 '/my/file/a', '/dir/somefile']
    assertInImproved(to_check[0][1][0], expected, self)
    assertInImproved(to_check[1][1][0], expected, self)
    trans_mock.getCounters.assert_called_once_with('TransformationFiles', ['Status'],
                                                    {'TransformationID': 78456})

  def test_sliced_limited_add_all(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK
    dataman_mock = Mock()
    util_mock = Mock()
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK(
        [({'Status': 'Processed'}, 4), ({'Status': 'junk'}, 6), ({'Status': 'Ignore_me'}, 8),
         ({'Status': 'Assigned'}, 6)])
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('SlicedLimited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 16
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    self.tfp.setInputData({
        '/my/file/here_abc': ['myTestSE'],
        '/my/file/other_file.txt': ['secretSE', 'StorageSE', 'TestSE'],
        '/my/file/newfile.pdf': ['TestSE'],
        '/my/file/a': [], '/dir/somefile': ['']})
    result = self.tfp.run()
    assertDiracSucceeds(result, self)
    expected = [('', ['/my/file/here_abc']), ('', ['/my/file/other_file.txt']),
                 ('', ['/my/file/newfile.pdf']), ('', ['/my/file/a']), ('', ['/dir/somefile'])]
    assertListContentEquals(result['Value'], expected, self)
    trans_mock.getCounters.assert_called_once_with('TransformationFiles', ['Status'],
                                                    {'TransformationID': 78456})

  def test_sliced_limited_too_many(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_OK
    dataman_mock = Mock()
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_OK([({'Status': 'Processed'}, 2)] * 14
                                                + [({'Status': 'Assigned'}, 1)] * 15
                                                + [({'Status': 'junk'}, 0)] * 4)
    self.tfp = TransformationPlugin('SlicedLimited', trans_mock, dataman_mock)
    self.tfp.params['MaxNumberOfTasks'] = 18
    self.tfp.params['TransformationID'] = 78456
    assertDiracFailsWith(self.tfp.run(), 'too many tasks', self)
    trans_mock.getCounters.assert_called_once_with('TransformationFiles', ['Status'],
                                                    {'TransformationID': 78456})

  def test_sliced_limited_getcounters_fails(self):
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    from DIRAC import S_ERROR
    dataman_mock = Mock()
    util_mock = Mock()
    trans_mock = Mock()
    trans_mock.getCounters.return_value = S_ERROR('my_test_getcount_err')
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('SlicedLimited', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Processed'
    self.tfp.params['MaxNumberOfTasks'] = 6
    self.tfp.params['TransformationID'] = 78456
    self.tfp.util = util_mock
    assertDiracFailsWith(self.tfp.run(), 'my_test_getcount_err', self)
    trans_mock.getCounters.assert_called_once_with('TransformationFiles', ['Status'],
                                                    {'TransformationID': 78456})

  def test_broadcast_processed_selectall(self):
    from DIRAC import S_OK
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    dataman_mock = Mock()
    trans_mock = Mock()
    util_mock = Mock()
    util_mock.fc.getFileDescendents.return_value = S_OK(
        {'Successful':
         {'/some/deep/dir/structure/file1.stdio': ['child'], '/file/dir/input1.txt': ['child'],
          '/dir/file.xml': ['child'], '/nodir.txt': ['child'], 'input_file.txt': ['child'],
          '/my/dir/pphys.ics': ['child']},
         'Failed': {}})
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('BroadcastProcessed', dataman_mock, trans_mock)
    self.tfp.params['Status'] = ''
    self.tfp.util = util_mock
    self.tfp.setInputData({'/some/deep/dir/structure/file1.stdio': ['myTestSE1'],
                             '/file/dir/input1.txt': ['other_SE'],
                             '/dir/file.xml': ['myTestSE1', 'SecondTestSE'],
                             '/nodir.txt': ['secretSE'],
                             'input_file.txt': ['SE_to_test'],
                             '/my/dir/pphys.ics': ['']})
    with patch('%s.TransformationPlugin._Broadcast' % MODULE_NAME, new=Mock(return_value=S_OK(98124))):
      assertDiracSucceedsWith_equals(self.tfp.run(), 98124, self)
      assertListContentEquals(self.tfp.data,
                               ['/some/deep/dir/structure/file1.stdio', '/file/dir/input1.txt',
                                 '/dir/file.xml', '/nodir.txt', 'input_file.txt', '/my/dir/pphys.ics'], self)
      util_mock.fc.getFileDescendents.assert_called_once()

  def test_broadcast_processed_flush(self):
    from DIRAC import S_OK
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    dataman_mock = Mock()
    trans_mock = Mock()
    self.tfp = TransformationPlugin('BroadcastProcessed', dataman_mock, trans_mock)
    self.tfp.params['Status'] = 'Flush'
    with patch('%s.TransformationPlugin._Broadcast' % MODULE_NAME, new=Mock(return_value=S_OK(8124))):
      assertDiracSucceedsWith_equals(self.tfp.run(), 8124, self)

  def test_broadcast_processed_getdesc_fails(self):
    from DIRAC import S_ERROR
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    dataman_mock = Mock()
    trans_mock = Mock()
    util_mock = Mock()
    util_mock.fc.getFileDescendents.return_value = S_ERROR('Descendant_test_err')
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('BroadcastProcessed', dataman_mock, trans_mock)
    self.tfp.params['Status'] = ''
    self.tfp.util = util_mock
    self.tfp.setInputData({'/some/deep/dir/structure/file1.stdio': ['myTestSE1'],
                             '/file/dir/input1.txt': ['other_SE'],
                             '/dir/file.xml': ['myTestSE1', 'SecondTestSE'],
                             '/nodir.txt': ['secretSE'],
                             'input_file.txt': ['SE_to_test'],
                             '/my/dir/pphys.ics': ['']})
    assertDiracFailsWith(self.tfp.run(), 'descendant_test_err', self)
    util_mock.fc.getFileDescendents.assert_called_once()

  def test_broadcast_processed_selectsome(self):
    from DIRAC import S_OK
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    dataman_mock = Mock()
    trans_mock = Mock()
    util_mock = Mock()
    util_mock.fc.getFileDescendents.return_value = S_OK(
        {'Successful':
         {'/some/deep/dir/structure/file1.stdio': ['child'], '/file/dir/input1.txt': [],
          '/dir/file.xml': ['child'], '/nodir.txt': ['child'], 'input_file.txt': ['child']},
         'Failed': {'/my/dir/pphys.ics': []}})
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('BroadcastProcessed', dataman_mock, trans_mock)
    self.tfp.params['Status'] = ''
    self.tfp.util = util_mock
    self.tfp.setInputData({'/some/deep/dir/structure/file1.stdio': ['myTestSE1'],
                             '/file/dir/input1.txt': ['other_SE'],
                             '/dir/file.xml': ['myTestSE1', 'SecondTestSE'],
                             '/nodir.txt': ['secretSE'],
                             'input_file.txt': ['SE_to_test'],
                             '/my/dir/pphys.ics': ['']})
    with patch('%s.TransformationPlugin._Broadcast' % MODULE_NAME, new=Mock(return_value=S_OK(98124))):
      assertDiracSucceedsWith_equals(self.tfp.run(), 98124, self)
      assertListContentEquals(self.tfp.data,
                               ['/some/deep/dir/structure/file1.stdio', '/dir/file.xml', '/nodir.txt',
                                 'input_file.txt'], self)
      util_mock.fc.getFileDescendents.assert_called_once()

  def test_broadcast_processed_checkchunking(self):
    from DIRAC import S_OK
    from ILCDIRAC.ILCTransformationSystem.Agent.TransformationPlugin import TransformationPlugin
    import copy
    files = {}
    for i in range(0, 599):
      files[('/file/dir/input%s.txt' % i)] = ['child']
    dataman_mock = Mock()
    trans_mock = Mock()
    util_mock = Mock()
    util_mock.fc.getFileDescendents.return_value = S_OK({'Successful': files, 'Failed': {}})
    util_mock.transClient = trans_mock
    self.tfp = TransformationPlugin('BroadcastProcessed', dataman_mock, trans_mock)
    self.tfp.params['Status'] = ''
    self.tfp.util = util_mock
    self.tfp.setInputData(copy.deepcopy(files))
    del self.tfp.data['/file/dir/input542.txt']
    with patch('%s.TransformationPlugin._Broadcast' % MODULE_NAME, new=Mock(return_value=S_OK(98124))):
      assertDiracSucceedsWith_equals(self.tfp.run(), 98124, self)
      expected = {}
      for i in range(0, 599):
        if i != 542:
          expected[('/file/dir/input%s.txt' % i)] = ['child']
      assertListContentEquals(self.tfp.data, expected, self)
    assertEqualsImproved(len(util_mock.fc.getFileDescendents.mock_calls), 3, self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Dummy module that prints out the workflow parameters.

:since: Mar 11, 2011

:author: sposs
"""

from __future__ import absolute_import
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from DIRAC import S_OK, S_ERROR, gLogger

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class DummyModule(ModuleBase):
  """Dummy module used to check Workflow Parameters (Parametric jobs check)"""

  def __init__(self):
    super(DummyModule, self).__init__()
    self.result = S_ERROR()

  def applicationSpecificInputs(self):
    """Resolve the parameters."""

    for key, val in self.workflow_commons.items():
      LOG.info("%s=%s" % (key, val))

    for key, val in self.step_commons.items():
      LOG.info("%s=%s" % (key, val))
    return S_OK()

  def execute(self):
    """Not much to do..."""
    self.result = self.resolveInputVariables()
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
PostGenSelection : Helper to filter generator selection
"""

from __future__ import absolute_import
import types

from DIRAC import S_OK, S_ERROR
from DIRAC.Core.Workflow.Parameter import Parameter

from ILCDIRAC.Interfaces.API.NewInterface.LCUtilityApplication import LCUtilityApplication
import six

__RCSID__ = "$Id$"


class PostGenSelection(LCUtilityApplication):
  """Helper to filter generator selection.

  Example:

  >>> postGenSel = PostGenSelection()
  >>> postGenSel.setNbEvtsToKeep(30)
  """

  def __init__(self, paramdict=None):

    self.numberOfEventsToKeep = 0
    super(PostGenSelection, self).__init__(paramdict)
    self._modulename = "PostGenSelection"
    self.appname = 'postgensel'
    self._moduledescription = 'Helper to filter generator selection'

  def setNbEvtsToKeep(self, numberOfEventsToKeep):
    """Set the number of events to keep in the input file.

    :param int numberOfEventsToKeep: number of events to keep in the input file. Must be inferior to the number of events.
    """
    self._checkArgs({'numberOfEventsToKeep': int})

    self.numberOfEventsToKeep = numberOfEventsToKeep
    return S_OK()

  def _applicationModule(self):
    m1 = self._createModuleDefinition()
    m1.addParameter(Parameter("NbEvtsKept", 0, "int", "", "", False, False, "Number of events to keep"))
    m1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    return m1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue('NbEvtsKept', self.numberOfEventsToKeep)
    moduleinstance.setValue('debug', self.debug)

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    """Checks that all needed parameters are set."""

    if not self.numberOfEventsToKeep:
      return S_ERROR('Number of events to keep was not given! Throw your brain to the trash and try again!')

    # res = self._checkRequiredApp() ##Check that job order is correct
    # if not res['OK']:
    #  return res

    return S_OK()

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()

  def _resolveLinkedStepParameters(self, stepinstance):
    if isinstance(self._linkedidx, six.integer_types):
      self._inputappstep = self._jobsteps[self._linkedidx]
    if self._inputappstep:
      stepinstance.setLink("InputFile", self._inputappstep.getType(), "OutputFile")
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Register a user in ILCDIRAC.

.. warning :: This script is disfavoured in view of the
      :class:`~DIRAC.ConfigurationSystem.Agent.VOMS2CSAgent.VOMS2CSAgent`

Register the user in the CS, the FileCatalog, and in the e-group.

For that last functionality to be work, you need to have the sections /Security/egroupAdmin and /Security/egroupPass
options in a dirac.cfg. Ideally, the local dirac.cfg (~/.dirac.cfg) can be used. It's also possible to pass the value
with ``-o /Security/egroupAdmin=something -o /Security/egroupPass=something``.

To figure out the commands available from WSDL::

  print client
"""

from __future__ import print_function
from __future__ import absolute_import
__RCSID__ = "$Id$"

try:
  import suds
  from suds.client import Client
except ImportError:
  print("suds module missing: Install it with [sudo] easy_install suds")
  raise

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import gLogger, gConfig, exit as dexit

from DIRAC import S_OK, S_ERROR


class _Params(object):
  """Parameter Object."""

  def __init__(self):
    self.uname = ''
    self.groups = []
    self.certDN = ''
    self.certCN = ''
    self.email = ''
    self.cernid = ''
    self.external = False

  def setUName(self, opt):
    self.uname = opt
    return S_OK()

  def setGroup(self, opt):
    self.groups = opt.split(",")
    return S_OK()

  def setDN(self, opt):
    self.certDN = opt
    return S_OK()

  def setCN(self, opt):
    self.certCN = opt
    return S_OK()

  def setEmail(self, opt):
    if not opt.find('@') > 0:
      return S_ERROR('Not a valid mail address')
    self.email = opt
    return S_OK()

  def setCERNID(self, opt):
    self.cernid = opt
    return S_OK()

  def setExternal(self, dummy_opt):
    self.external = True
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch("U:", "UserName=", "DIRAC user name", self.setUName)
    Script.registerSwitch("G:", "Groups=", "DIRAC groups in which to add the new user, comma separated", self.setGroup)
    Script.registerSwitch("D:", "DN=", "user DN", self.setDN)
    Script.registerSwitch("C:", "CN=", "user CN (or CA)", self.setCN)
    Script.registerSwitch("E:", "Email=", "User mail", self.setEmail)
    Script.registerSwitch("", "CCID=", "CERN CC user ID (if any)", self.setCERNID)
    Script.registerSwitch("X", "external", "set if user is external, no checking of the PhoneBook", self.setExternal)
    Script.setUsageMessage(
        """%s -U <username> -G <ilc_user,private_pilot,...> -D"<DN>" -C"<CN>" -E <email>""" %
        Script.scriptName)


def addUserToCS(clip, userProps):
  """Add the user to the CS, return list of errors."""
  from DIRAC.Interfaces.API.DiracAdmin import DiracAdmin
  diracAdmin = DiracAdmin()
  exitCode = 0
  errorList = []

  if not diracAdmin.csModifyUser(clip.uname, userProps, createIfNonExistant=True)['OK']:
    errorList.append(("add user", "Cannot register user: '%s'" % clip.uname))
    exitCode = 255
  else:
    result = diracAdmin.csCommitChanges()
    if not result['OK']:
      errorList.append(("commit", result['Message']))
      exitCode = 255
  for error in errorList:
    gLogger.error("%s: %s" % error)
  if exitCode:
    dexit(exitCode)


def addUserToFC(clip):
  """Add the user to the filecatalog Try to figure out in which VOs the user must be, and create the catalog entries accordingly."""
  from DIRAC.ConfigurationSystem.Client.Helpers import Registry
  from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
  from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
  fc = FileCatalogClient()
  res = fc.addUser(clip.uname)
  if not res['OK']:
    gLogger.error("Failed to add user to FC, but it does not really matter:", res['Message'])
    gLogger.error("Add by hand (in the FC-CLI: user add %s)" % (clip.uname))

  bpath = ''
  for grp in clip.groups:
    bpath = '/'
    voName = Registry.getVOForGroup(grp)
    if not voName:
      gLogger.error("NO VO for group", grp)
      continue
    bpath += voName + "/"
    lfnprefix = Operations(vo=voName).getValue("LFNUserPrefix")
    if lfnprefix:
      bpath += lfnprefix + "/"
    bpath += clip.uname[0] + "/" + clip.uname

    res = fc.createDirectory(bpath)
    if not res['OK']:
      gLogger.error(res['Message'])
      continue

    res = fc.changePathGroup({bpath: grp}, False)
    if not res['OK']:
      gLogger.error(res['Message'])

    res = fc.changePathOwner({bpath: clip.uname}, False)
    if not res['OK']:
      gLogger.error(res['Message'])

    res = fc.setMetadata(bpath, {"Owner": clip.uname})
    if not res['OK']:
      gLogger.error(res['Message'])


def addUserToEgroup(clip):
  """Add user to e-group."""
  login = gConfig.getValue("/Security/egroupAdmin", "").strip('"')
  pwd = gConfig.getValue("/Security/egroupPass", "").strip('"')
  url = 'https://foundservices.cern.ch/ws/egroups/v1/EgroupsWebService/EgroupsWebService.wsdl'
  if not (login and pwd):
    gLogger.warn("Missing configuration parameters: username or password for WSDL interactions")
    gLogger.warn("Add options: -o /Security/egroupAdmin=<cernusername> -o /Security/egroupPass=<password>")
    gLogger.error("User registration in e-group must be done manually")
    return
  try:
    client = Client(url=url, username=login, password=pwd)
    # gLogger.notice(client)
  except suds.transport.TransportError as exc:
    gLogger.error("Failed to get the WSDL client:%s" % exc)
    gLogger.error("User registration in e-group must be done manually")
    return
  except BaseException:
    gLogger.error("Something unexpected happened with the suds client, aborting")
    return

  if clip.external:
    sudsUser = client.factory.create("ns0:MemberType")
    sudsUser['Type'] = 'External'
    sudsUser['Email'] = clip.email
    userl = [sudsUser]
  else:
    user = getUserInfoFromPhonebook(client, clip)
    userl = [user]
  res = client.service.AddEgroupMembers('ilc-dirac', False, userl)
  if hasattr(res, 'warnings'):
    gLogger.notice(res.warnings)


def getUserInfoFromPhonebook(client, clip):
  """return user information from the phonebook."""
  sudsUser = client.factory.create("ns0:MemberType")
  comm = "phonebook --login %s --terse firstname --terse surname --terse ccid --terse email" % clip.uname
  from DIRAC.Core.Utilities.Subprocess import shellCall
  res = shellCall(0, comm)
  if not res['OK']:
    gLogger.error("Failed getting user info:", res['Message'])
    gLogger.error("Please add user in e-group by hand.")
    dexit(1)
  if res['Value'][0]:
    gLogger.error("phonebook command returned an error:", res['Value'][2])
    gLogger.error("Please add user in e-group by hand.")
    dexit(1)
  output = res['Value'][1]
  if output:
    output = output.split("\n")
    if len(output) > 2:
      gLogger.error("This username somehow has more than one account, please choose the right one and register by hand")
      gLogger.error("%s" % output)
      dexit(1)
    user_fname = output[0].split(";")[0]  # firstname
    user_sname = output[0].split(";")[1]  # surname
    phoneBookEmail = output[0].split(";")[3]  # email
    if phoneBookEmail != clip.email and clip.email:
      gLogger.error("Email for user (%s) does not match the email given in the command line (%s))"
                     % (phoneBookEmail, clip.email))
      dexit(1)
    else:
      clip.email = phoneBookEmail
    sudsUser['PrimaryAccount'] = clip.uname.upper()
    sudsUser['ID'] = output[0].split(";")[2]  # CCID
    sudsUser['Type'] = 'Person'
    sudsUser['Name'] = '%s, %s' % (user_sname.upper(), user_fname)
    sudsUser['Email'] = output[0].split(";")[3]  # email
  else:
    gLogger.notice("User '%s' does not appear to be in the CERN phonebook" % clip.uname)
    if not clip.email:
      gLogger.error("Email address for external user is not given, please add -E<address>")
      dexit(1)

    sudsUser['Type'] = 'External'
    sudsUser['Email'] = clip.email

  return sudsUser


@DIRACScript()
def main():
  """Add user to configuration service and other things."""
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine()
  if not (clip.certCN and clip.groups and clip.certDN and clip.uname):
    gLogger.error("Username, DN, CN, and groups have to be given")
    Script.showHelp()
  gLogger.notice("Add User to Egroup")
  addUserToEgroup(clip)
  if not clip.email:
    gLogger.fatal("No email defined and not found in phonebook, you have to provide it: -E<email>")
    dexit(1)
  userProps = {'DN': clip.certDN, 'Email': clip.email, 'CN': clip.certCN, 'Groups': clip.groups}
  gLogger.notice("Add User to CS")
  addUserToCS(clip, userProps)
  gLogger.notice("Add User to FC")
  addUserToFC(clip)
  gLogger.notice("Done")


if __name__ == "__main__":
  main()
  dexit(0)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Download the output data for the jobs in the repository file created during job submission.

The repository file is defined when creating the :class:`~ILCDIRAC.Interfaces.API.DiracILC.DiracILC` instance.

Options:

  -r repoLocation       Path to repository file

:since: Mar 24, 2010
:author: sposs
"""
from __future__ import absolute_import
from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, exit as dexit

__RCSID__ = "$Id$"


class _Params(object):
  """CLI params class."""

  def __init__(self):
    self.repo = ''

  def setRepo(self, val):
    self.repo = val
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch("r:", "repo=", "repo file", self.setRepo)
    # Define a help message
    Script.setUsageMessage('\n'.join([__doc__,
                                         'Usage:',
                                         '  %s [option|cfgfile] ' % Script.scriptName]))


@DIRACScript()
def main():
  cliParams = _Params()
  cliParams.registerSwitches()
  Script.parseCommandLine(ignoreErrors=False)
  if not cliParams.repo:
    Script.showHelp()
    dexit(2)
  from DIRAC.Interfaces.API.Dirac import Dirac

  dirac = Dirac(True, cliParams.repo)

  exitCode = 0
  dirac.monitorRepository(False)
  dirac.retrieveRepositoryData()

  dexit(exitCode)


if __name__ == "__main__":
  main()



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the ILCDIRAC Application module."""

from __future__ import print_function
from __future__ import absolute_import
import inspect
import unittest
from mock import MagicMock as Mock, patch

from DIRAC import gLogger
from ILCDIRAC.Interfaces.API.NewInterface.Application import Application
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith_equals

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Application'

gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)

# pylint: disable=protected-access


class TestApplication(unittest.TestCase):
  """tests for the Application base interface."""

  def setUp(self):
    self.app = Application()

  def tearDown(self):
    pass

  def test_setParam_1(self):
    """test _setParam function."""
    params = {'Version': '01-00'}
    ret = self.app._setparams(params)
    self.assertTrue(ret['OK'], ret.get('Message', ''))
    self.assertEqual(self.app.version, params['Version'])

  def test_setParam_Fail_1(self):
    """test _setParam function, make sure getattr throws correct exception."""
    params = {'NotVersion': '01-00'}
    ret = self.app._setparams(params)
    self.assertTrue(ret['OK'], ret.get('Message', ''))


# pylint: disable=protected-access
class AppTestCase(unittest.TestCase):
  """Test the ILCDIRAC Application class."""

  def setUp(self):
    """set up the objects."""
    self.app = Application({})

  def test_repr(self):
    app1 = Application({})
    app1.appname = 'MyTestApp'
    app1.version = 'v1'
    assertEqualsImproved(app1.__repr__(), 'MyTestApp v1', self)
    app2 = Application({})
    app2.appname = 'secondTestApp'
    app2.version = ''
    assertEqualsImproved(app2.__repr__(), 'secondTestApp', self)

  def test_setparams(self):
    self.app._setparams({'Name': 'mytestapp', 'Version': 'tv1', 'SteeringFile': 'test.steer',
                           'LogFile': 'mylog.log', 'OutputFile': 'test.out', 'OutputSE': 'CERN-TEST',
                           'InputFile': 'test.in'})
    app = self.app
    assertEqualsImproved((app.appname, app.version, app.steeringFile, app.logFile, app.outputFile,
                            app.outputSE, app.inputFile), ('mytestapp', 'tv1', 'test.steer', 'mylog.log',
                                                             'test.out', 'CERN-TEST', 'test.in'), self)
    self.assertFalse(app._errorDict)

  def test_setparams_empty(self):
    assertDiracSucceeds(self.app._setparams({}), self)

  def test_setparams_wrongtype(self):
    assertDiracSucceeds(self.app._setparams({'Name': 134, 'Version': {False: True},
                                                'OutputSE': True}), self)
    assertEqualsImproved(len(self.app._errorDict['_checkArgs']), 3, self)

  def test_setparams_nonexistant_attribute(self):
    log_mock = Mock()
    with patch.object(inspect.getmodule(Application), 'LOG', new=log_mock):
      assertDiracSucceeds(self.app._setparams({'ThisAttributeIsntPartOfApplication': 'someStuff'}), self)
    self.assertTrue(log_mock.error.called)

  def test_getparamsdict(self):
    self.app = Application({'Name': '', 'Version': 'v18', 'SteeringFile': 'testSteer',
                              'LogFile': 'log.test', 'OutputFile': 'out.test'})
    result = self.app._getParamsDict()
    assertDiracSucceedsWith_equals(result, {'version': 'v18', 'steeringFile': 'testSteer',
                                              'logFile': 'log.test', 'outputFile': 'out.test'}, self)

# Most setters already tested via _setparams tests which call setX
  def test_getinputfromapp(self):
    app_mock = Mock()
    app_mock.appname = 'myCoolApp'
    self.assertFalse(self.app._inputapp)
    assertDiracSucceeds(self.app.getInputFromApp(app_mock), self)
    assertEqualsImproved(self.app._inputapp, [app_mock], self)

  def test_listattributes(self):
    log_mock = Mock()
    with patch.object(inspect.getmodule(Application), 'LOG', new=log_mock):
      self.app.setDebug(False)
      self.app.setExtraCLIArguments('my_custom_option other_option cl_argument')
      self.app.listAttributes()
    log_mock.notice.assert_any_call('  debug: Not defined')
    log_mock.notice.assert_any_call('  extraCLIArguments: my_custom_option other_option cl_argument')

  def test_setters(self):
    self.app.setSteeringFile('/nonexistant/dir/mysteerfile')
    self.assertFalse(self.app.inputSB)
    self.app.setSteeringFile('lfn:/mydir/mysteerfile')
    assertEqualsImproved(self.app.inputSB, ['lfn:/mydir/mysteerfile'], self)
    self.app.proparameters = {}
    self.app.detectortype = 'mytestdetector'
    self.app.datatype = 'mytestdatatype'
    assertDiracSucceeds(self.app.setOutputFile('mytestoutputfile', 'mytestpath'), self)
    assertEqualsImproved((self.app.prodparameters, self.app.outputFile, self.app.outputPath),
                          ({'mytestoutputfile': {'detectortype': 'mytestdetector',
                                                     'datatype': 'mytestdatatype'}}, 'mytestoutputfile',
                            'mytestpath'), self)
    self.app.inputSB = []
    assertDiracFailsWith(self.app.setInputFile(True),
                          'Problem with ILCDIRAC.Interfaces.API.NewInterface.Application.setInputFile', self)
    self.app.setInputFile('lfn:/mydir/input.file')
    assertEqualsImproved(self.app.inputSB, ['lfn:/mydir/input.file'], self)
    self.app.inputSB = []
    self.app.setInputFile(['/invalid/dir/myfile.txt', 'lfn:/valid/dir/myfile.txt'])
    assertEqualsImproved(self.app.inputSB, ['lfn:/valid/dir/myfile.txt'], self)

  def test_private_methods(self):
    with patch.object(self.app, '_applicationModule', new=Mock(return_value=Mock())) as module_mock, \
        patch.object(self.app, '_applicationModuleValues', new=Mock(return_value=Mock())) as module_mock:
      self.app._setApplicationModuleAndParameters(Mock())
      self.assertTrue(module_mock.called)
    self.app._setOutputComputeDataList(Mock())
    self.app._getComputeOutputDataListModule()
    with self.assertRaises(NotImplementedError):
      self.app._applicationModule()
    self.app._applicationModuleValues(None)
    assertDiracFailsWith(self.app._userjobmodules(None), 'not implemented', self)
    assertDiracFailsWith(self.app._prodjobmodules(None), 'not implemented', self)
    assertDiracSucceeds(self.app._checkConsistency(), self)
    self.app._inputapp = ['myapp', 'failonthis']
    self.app._jobapps = ['myapp', 'irrelevant_app', 'other_app']
    assertDiracFailsWith(self.app._checkRequiredApp(), 'job order not correct', self)


def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestApplication)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()



"""Retrieve ECalToHad and HCalToHad from Calibration.txt file."""

from __future__ import print_function
from __future__ import absolute_import
import sys
from Helper_Functions import find_between


Calibration_File_And_Path = sys.argv[1]
Energy_To_Calibrate = sys.argv[2]
Initial_Calibration_Constant = sys.argv[3]
Mean_Or_Calibration_Constant = sys.argv[4]

# Mean is the last value for Mean of Gaussian fit written to the Calibration.txt file.

Mean = ''

with open(Calibration_File_And_Path, 'r') as f:
    searchlines = f.readlines()
    for line in searchlines:
        if 'ECalToEM Mean' in line:
            Mean = float(find_between(line, ' : ', ' : '))

if Mean_Or_Calibration_Constant == 'Mean':
    print (Mean)

elif Mean_Or_Calibration_Constant == 'Calibration_Constant':
    print (float(Energy_To_Calibrate) * float(Initial_Calibration_Constant) / float(Mean))

    Calibration_Text = '_____________________________________________________________________________________' + '\n'
    Calibration_Text += 'EM_Extract.py retrieving ECTE and HCTE from PandoraPFA calibration program.' + '\n'
    Calibration_Text += 'Photon Energy To Calibrate                         : ' + str(Energy_To_Calibrate) + '\n'
    Calibration_Text += 'Initial Calibration Constant                       : ' + \
        str(Initial_Calibration_Constant) + '\n'
    Calibration_Text += 'Mean                                               : ' + str(Mean) + '\n\n'
    Calibration_Text += 'Updated => ECalToHad = HCalToHad =                 : ' + \
        str(float(Energy_To_Calibrate) * float(Initial_Calibration_Constant) / float(Mean)) + '\n\n'

    with open(Calibration_File_And_Path, 'a') as myfile:
        myfile.write(Calibration_Text)

else:
    print ('Please select Mean or Calibration_Constant to extract.')



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Sub class of TransformationPlugin to allow for extending the ILD sim jobs."""

from __future__ import absolute_import
from DIRAC.TransformationSystem.Agent.TransformationPlugin import TransformationPlugin as DTP
from DIRAC.Core.Utilities.List import breakListIntoChunks
from DIRAC import S_OK, S_ERROR


class TransformationPlugin(DTP):
  """This plugin is ONLY used when willing to limit the number of tasks to a certain number of files."""

  def __init__(self, plugin, transClient=None, dataManager=None):
    super(TransformationPlugin, self).__init__(plugin, transClient, dataManager)

  def _Limited(self):
    """Limit the number of tasks created to the MaxNumberOfTasks.

    Get the total number of tasks submitted. Check if that's bigger than the
    MaxNumberOfTasks Extend by the number of tasks if needed to reach MaxNumberOfTasks.
    """
    max_tasks = self.params['MaxNumberOfTasks']
    res = self.util.transClient.getCounters('TransformationFiles', ['Status'],
                                             {'TransformationID': self.params['TransformationID']})
    if not res['OK']:
      return res
    total_used = 0
    for statustup in res['Value']:
      if statustup[0]['Status'] in ['Assigned', 'Processed']:
        total_used += statustup[1]
    if total_used >= max_tasks and max_tasks > 0:
      return S_OK([])
    res = self.util.groupByReplicas(self.data, self.params['Status'])
    if not res['OK']:
      return res
    newTasks = []
    for _se, lfns in res['Value']:
      newTasks.append(('', lfns))
      total_used += 1
      if total_used >= max_tasks and max_tasks > 0:
        break
    return S_OK(newTasks)

  def _Sliced(self):
    """Handle the slicing, in fact do nothing particular."""

    lfns = self.data
    # self.data is the dict of replicas for each file d[lfn]=[replicas]
    newTasks = []
    for lfn in lfns.keys():
      newTasks.append(('', [lfn]))

    return S_OK(newTasks)

  def _SlicedLimited(self):
    """For the Sliced productions.

    Limit the number of tasks created to the MaxNumberOfTasks.
    Get the total number of tasks submitted. Check if that's bigger than the MaxNumberOfTasks.
    Extend by the number of tasks if needed to reach MaxNumberOfTasks.
    """
    max_tasks = self.params['MaxNumberOfTasks']
    res = self.util.transClient.getCounters('TransformationFiles', ['Status'],
                                             {'TransformationID': self.params['TransformationID']})
    if not res['OK']:
      return res
    total_used = 0
    for statustup in res['Value']:
      if statustup[0]['Status'] in ['Assigned', 'Processed']:
        total_used += statustup[1]
    if total_used >= max_tasks and max_tasks > 0:
      return S_ERROR('Too many tasks for this transformation')
    lfns = self.data
    newTasks = []
    for lfn in lfns.keys():
      newTasks.append(('', [lfn]))
      total_used += 1
      if total_used >= max_tasks and max_tasks > 0:
        break
    return S_OK(newTasks)

  def _BroadcastProcessed(self):
    """this plug-in only creates tasks for files which have descendents."""
    transformationStatus = self.params['Status']
    if transformationStatus in ('Flush', ):
      self.util.logInfo("Flushing transformation, passing all files on")
      return self._Broadcast()

    inputFiles = self.data
    self.util.logInfo("Number of input files before selection: %d " % len(inputFiles))

    # query only a maximum of 200 files in one go
    inputFileLists = breakListIntoChunks(list(inputFiles.keys()), 200)

    for ifList in inputFileLists:
      resDesc = self.util.fc.getFileDescendents(ifList, depths=1)
      self.util.logDebug("Result from getFileDescendents: %s " % resDesc)
      if not resDesc['OK']:
        return resDesc
      descendents = resDesc['Value']

      for lfn in ifList:
        if lfn not in descendents['Successful']:
          self.util.logDebug("Removed: %s, not in succesful " % lfn)
          inputFiles.pop(lfn, None)
        elif not descendents['Successful'][lfn]:
          self.util.logDebug("Removed: %s no descendents" % lfn)
          inputFiles.pop(lfn, None)

      if descendents['Failed']:
        self.util.logWarn("Failed getDescendents: %s " % descendents['Failed'])

    self.util.logInfo("Number of input files after selection: %d " % len(inputFiles))

    self.data = inputFiles
    return self._Broadcast()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""This contains the ILC specific API plugins for DIRAC.

:author: S. Poss
:since: Jan 23, 2010
"""
__RCSID__ = "$Id$"



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Services for Overlay System."""

from __future__ import absolute_import

from DIRAC import S_OK
from DIRAC.Core.DISET.RequestHandler import RequestHandler

from ILCDIRAC.OverlaySystem.DB.OverlayDB import OverlayDB

__RCSID__ = "$Id$"

# pylint: disable=unused-argument,no-self-use, global-statement

# This is a global instance of the OverlayDB class
OVERLAY_DB = False


def initializeOverlayHandler(serviceInfo):
  """Global initialize for the Overlay service handler."""
  global OVERLAY_DB
  OVERLAY_DB = OverlayDB()
  return S_OK()


class OverlayHandler(RequestHandler):
  """Service for Overlay."""
  types_canRun = [(str,)]

  def export_canRun(self, site):
    """Check if current job can access the data."""
    return OVERLAY_DB.canRun(site)

  types_jobDone = [(str,)]

  def export_jobDone(self, site):
    """report that a given job is done downloading the files at a given site."""
    return OVERLAY_DB.jobDone(site)

  types_getJobsAtSite = [(str,)]

  def export_getJobsAtSite(self, site):
    """Get the jobs running at a given site."""
    return OVERLAY_DB.getJobsAtSite(site)

  types_getSites = []

  def export_getSites(self):
    """Get all sites registered."""
    return OVERLAY_DB.getSites()

  types_setJobsAtSites = [dict]

  def export_setJobsAtSites(self, sitedict):
    """Set the number of jobs running at each site:

    called from the ResetCounter agent
    """
    return OVERLAY_DB.setJobsAtSites(sitedict)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Module to concatenate LCIO files.

:author: S. Poss
:since: Mar 09, 2012
"""

from __future__ import absolute_import
import os

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Utilities.Subprocess import shellCall

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import getNewLDLibs
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename, resolveIFpaths
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class StdHepSplit(ModuleBase):
  """StdHep split module, split StdHep files using A.

  Miyamoto's HepSplit utility
  """

  def __init__(self):

    super(StdHepSplit, self).__init__()

    self.STEP_NUMBER = ''
    self.result = S_ERROR()
    self.nbEventsPerSlice = 0
    self.InputFile = []
    # Step parameters
    self.prod_outputdata = []
    #
    self.listoutput = {}
    self.OutputFile = []
    LOG.info("%s initialized" % (self.__str__()))
    self.maxRead = 0

  def applicationSpecificInputs(self):
    """Resolve LCIO concatenate specific parameters, called from ModuleBase."""

    if not self.OutputFile:
      return S_ERROR('No output file defined')

    if 'IS_PROD' in self.workflow_commons and self.workflow_commons["IS_PROD"]:
      if 'ProductionOutputData' in self.workflow_commons:
        self.prod_outputdata = self.workflow_commons['ProductionOutputData'].split(";")
        for obj in self.prod_outputdata:
          if obj.lower().count("_gen_"):
            self.OutputFile = os.path.basename(obj)
      else:
        self.OutputFile = getProdFilename(self.OutputFile,
                                          int(self.workflow_commons["PRODUCTION_ID"]),
                                          int(self.workflow_commons["JOB_ID"]))

    if not len(self.InputFile) and len(self.InputData):
      for files in self.InputData:
        if files.lower().find(".stdhep") > -1:
          self.InputFile.append(files)

    if 'listoutput' in self.step_commons:
      if len(self.step_commons['listoutput']):
        self.listoutput = self.step_commons['listoutput'][0]

    return S_OK('Parameters resolved')

  def execute(self):
    """Execute the module, called by JobAgent."""
    # Get input variables

    self.result = self.resolveInputVariables()
    # Checks

    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')

    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result["Message"])
      return self.result

    if len(self.InputFile):
      res = resolveIFpaths(self.InputFile)
      if not res['OK']:
        LOG.error("Cannot find input file")
        self.setApplicationStatus('StdHepSplit: missing input stdhep file')
        return S_ERROR('Missing stdhep file!')
      runonstdhep = res['Value'][0]
    else:
      LOG.warn("No files found to split")
      return S_OK("No files found to process")

    prefix = ''
    if self.OutputFile:
      prefix = self.OutputFile.split('.stdhep')[0]

    else:
      prefix = "this_split"
    # because we need to make sure the files end up in the base directory at the end
    self.OutputFile = prefix

    LOG.info("Will rename all files using '%s' as base." % prefix)

    # Setting up script
    res = getSoftwareFolder(self.platform, "stdhepsplit", self.applicationVersion)
    if not res['OK']:
      LOG.error("Failed to find the software")
      self.setApplicationStatus('StdHepSplit: Could not find neither local area not shared area install')
      return res

    mysplitDir = res['Value']
    new_ld_lib = getNewLDLibs(self.platform, "stdhepsplit", self.applicationVersion)
    LD_LIBRARY_PATH = os.path.join(mysplitDir, "lib") + ":" + new_ld_lib

    maxReadString = ''
    # hepsplit has a off-by-one error so we add 1 to max read to actually read maxRead events
    if self.maxRead:
      self.maxRead += 1
      maxReadString = " --maxread %s" % self.maxRead

    scriptContent = """
#!/bin/sh

################################################################################
# Dynamically generated script by LCIOConcatenate module                       #
################################################################################

declare -x LD_LIBRARY_PATH=%s

%s/hepsplit --infile %s --nw_per_file %s --outpref %s%s

exit $?

""" % (
        LD_LIBRARY_PATH,
        mysplitDir,
        runonstdhep,
        self.nbEventsPerSlice,
        prefix,
        maxReadString,
        )

    # Write script to file

    scriptPath = 'StdHepSplit_%s_Run_%s.tcl' % (self.applicationVersion, self.STEP_NUMBER)

    if os.path.exists(scriptPath):
      os.remove(scriptPath)

    script = open(scriptPath, 'w')
    script.write(scriptContent)
    script.close()

    # Setup log file for application stdout

    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    # Run code

    os.chmod(scriptPath, 0o755)

    command = '"./%s"' % (scriptPath)

    self.setApplicationStatus('StdHepSplit %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''

    self.result = shellCall(0,
                             command,
                             callbackFunction=self.redirectLogOutput,
                             bufferLimit=20971520
                           )

    resultTuple = self.result['Value']
    status = resultTuple[0]

    if not os.path.exists(self.applicationLog):
      LOG.error("Cannot access log file, cannot proceed")
      return S_ERROR("Failed reading the log file")

    with open(self.applicationLog, "r") as logf:
      numberofeventsdict = {}
      fname = ''
      for line in logf:
        line = line.rstrip()
        if line.count('Open output file'):
          fname = line.split()[-1].rstrip().rstrip("\0")
          numberofeventsdict[fname] = 0
        elif line.count("Record") and not line.count('Output Begin Run'):
          #print line
          val = line.split("=")[1].rstrip().lstrip()
          if val != '0':
            numberofeventsdict[fname] = int(val)

    LOG.verbose("numberofeventsdict dict: %s" % numberofeventsdict)

    # Now update the workflow_commons dict with the relation between filename and number of events:
    # needed for the registerOutputData
    self.workflow_commons['file_number_of_event_relation'] = numberofeventsdict
    if self.listoutput:
      outputlist = []
      for of in numberofeventsdict:
        item = {}
        item['outputFile'] = of
        item['outputPath'] = self.listoutput['outputPath']
        item['outputDataSE'] = self.listoutput['outputDataSE']
        outputlist.append(item)
      self.step_commons['listoutput'] = outputlist

    # Not only the step_commons must be updated
    if 'ProductionOutputData' in self.workflow_commons:
      proddata = self.workflow_commons['ProductionOutputData'].split(";")
      finalproddata = []
      this_split_data = ''
      for item in proddata:
        if not item.count(prefix):
          finalproddata.append(item)
        else:
          this_split_data = item
      path = os.path.dirname(this_split_data)
      for of in numberofeventsdict:
        finalproddata.append(os.path.join(path, of))
      self.workflow_commons['ProductionOutputData'] = ";".join(finalproddata)

    LOG.info("Status after the application execution is %s" % str(status))
    if status == 2:
      LOG.info("Reached end of input file")
      status = 0

    self.listDir()
    return self.finalStatusReport(status)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""mixin for slic/lcsim commong functions."""

from __future__ import absolute_import
import os
import six.moves.urllib.request
import six.moves.urllib.parse
import six.moves.urllib.error
import zipfile
from DIRAC import S_OK, S_ERROR

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import unzip_file_into_dir


class CompactMixin(object):
  """mixin for detector geometry files used by slic and lcsim."""

  def getDetectorModel(self):
    """retrieve detector model from web."""
    retDown = self.downloadDetectorZip()
    if not retDown['OK']:
      return retDown

    detectorModel = self.detectorModel[:-4] if self.detectorModel.endswith(".zip") else self.detectorModel
    if not os.path.exists(detectorModel + ".zip"):
      self.log.error('Detector model %s was not found neither locally nor on the web, exiting' % detectorModel)
      return S_ERROR('Detector model %s was not found neither locally nor on the web, exiting' % detectorModel)
    try:
      unzip_file_into_dir(open(detectorModel + ".zip", 'rb'), os.getcwd())
    except (RuntimeError, OSError, zipfile.BadZipfile) as err:  # RuntimeError is for zipfile
      os.unlink(detectorModel + ".zip")
      self.log.error('Failed to unzip detector model: ', str(err))
      return S_ERROR('Failed to unzip detector model')
    # unzip detector model
    # self.unzip_file_into_dir(open(detectorModel+".zip", 'rb'),os.getcwd())
    return S_OK()

  def downloadDetectorZip(self):
    """download the detector zip file."""
    detector_urls = self.ops.getValue('/SLICweb/SLICDetectorModels', [''])
    if len(detector_urls[0]) < 1:
      self.log.error('Could not find in CS the URL for detector model')
      return S_ERROR('Could not find in CS the URL for detector model')

    detectorModel = self.detectorModel[:-4] if self.detectorModel.endswith(".zip") else self.detectorModel
    if not os.path.exists(detectorModel + ".zip"):
      for detector_url in detector_urls:
        try:
          six.moves.urllib.request.urlretrieve("%s%s" % (detector_url, detectorModel + ".zip"),
                             detectorModel + ".zip")
        except IOError as e:
          self.log.error("Download of detector model failed", str(e))
          continue
    return S_OK()



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test SLICPandora module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLICPandora
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertInImproved

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.SLICPandora'

# pylint: disable=protected-access


class SLICPandoraTestCase(unittest.TestCase):
  """Base class for the SLICPandora test cases."""

  def setUp(self):
    """set up the objects."""
    self.slic = SLICPandora({})
    self.slic.version = 941

  def test_setters(self):
    with patch('os.path.exists', new=Mock(return_value=True)):
      self.slic.setDetectorModel(123)
      self.slic.setPandoraSettings(94)
    self.slic.setStartFrom('soirmgf')
    assertInImproved('_checkArgs', self.slic._errorDict, self)
    (det_found, start_found, pand_found) = (False, False, False)
    for err in self.slic._errorDict['_checkArgs']:
      if 'detectorModel' in err:
        det_found = True
      elif 'startfrom' in err:
        start_found = True
      elif 'pandoraSettings' in err:
        pand_found = True
    assertEqualsImproved((True, True, True), (det_found, start_found, pand_found), self)
    assertEqualsImproved(len(self.slic.inputSB), 2, self)

  def test_setdetectormodel(self):
    self.slic.setDetectorModel('lfn:/some/path/testDetectorv2')
    assertEqualsImproved((self.slic.detectorModel, self.slic.inputSB),
                          ('lfn:/some/path/testDetectorv2', ['lfn:/some/path/testDetectorv2']), self)
    self.assertFalse(self.slic._errorDict)

  def test_setdectormodel_notfound(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.slic.setDetectorModel('invalid_detector')
      assertEqualsImproved((self.slic.detectorModel, self.slic.inputSB),
                            ('invalid_detector', []), self)
      self.assertFalse(self.slic._errorDict)

  def test_setpandorasettings(self):
    self.slic.setPandoraSettings('lfn:/some/path/my_pand_settings.txt')
    assertEqualsImproved((self.slic.pandoraSettings, self.slic.inputSB),
                          ('lfn:/some/path/my_pand_settings.txt', ['lfn:/some/path/my_pand_settings.txt']),
                          self)
    self.assertFalse(self.slic._errorDict)

  def test_setpandorasettings_notfound(self):
    with patch('os.path.exists', new=Mock(return_value=False)):
      self.slic.setPandoraSettings('invalid_settings.xml')
      assertEqualsImproved((self.slic.pandoraSettings, self.slic.inputSB),
                            ('invalid_settings.xml', []), self)
      self.assertFalse(self.slic._errorDict)

  def test_applicationmodule(self):
    result = self.slic._applicationModule()
    self.assertIsNotNone(result)

  def test_applicationmodulevalues(self):
    module_mock = Mock()
    self.slic._applicationModuleValues(module_mock)
    self.assertTrue(module_mock.setValue.called)
    assertEqualsImproved(len(module_mock.mock_calls), 4, self)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.slic._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.slic._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.slic._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.slic._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_resolveparameters(self):
    step_mock = Mock()
    assertDiracSucceeds(self.slic._resolveLinkedStepParameters(step_mock()), self)
    assertDiracSucceeds(self.slic._checkWorkflowConsistency(), self)

  def test_resolveparameters_setlink(self):
    step_mock = Mock()
    self.slic._linkedidx = 1
    self.slic._jobsteps = [None, Mock()]
    assertDiracSucceeds(self.slic._resolveLinkedStepParameters(step_mock()), self)

  def test_checkconsistency(self):
    import inspect
    self.slic.version = 'v2.4'
    self.slic.steeringFile = 'myTestSteerFile.txt'
    self.slic.pandoraSettings = 'mysettings.xml'
    self.slic.startFrom = False
    self.slic._jobtype = 'notUser'
    with patch('os.path.exists', new=Mock(side_effect=[False])), \
         patch.object(inspect.getmodule(SLICPandora), 'Exists', new=Mock(return_value=S_OK('success!'))):
      assertDiracSucceeds(self.slic._checkConsistency('myTestJob'), self)

  def test_checkconsistency_noversion(self):
    self.slic.version = None
    assertDiracFailsWith(self.slic._checkConsistency(), 'no version found', self)

  def test_checkconsistency_steering_Exists_fails(self):
    import inspect
    self.slic.version = 'v2.4'
    self.slic.steeringFile = 'myTestSteerFile.txt'
    self.slic.trackingStrategy = 'myTestStrat'
    self.slic.detectorModel = 'myDetModel.wrongFormat'
    with patch('os.path.exists', new=Mock(side_effect=[False])), \
         patch.object(inspect.getmodule(SLICPandora), 'Exists', new=Mock(return_value=S_ERROR('Exists_test_err'))):
      assertDiracFailsWith(self.slic._checkConsistency('myTestJob'), 'exists_test_err', self)

  def test_checkconsistency_nosettings(self):
    self.slic.pandoraSettings = None
    self.slic.steeringFile = None
    assertDiracFailsWith(self.slic._checkConsistency(), 'pandorasettings not set', self)

  def test_checkconsistency_othercase(self):
    import inspect
    self.slic.version = 'v2.4'
    self.slic.steeringFile = 'myTestSteerFile.txt'
    self.slic.pandoraSettings = 'mysettings.xml'
    self.slic.startFrom = True
    self.slic._jobtype = 'User'
    with patch('os.path.exists', new=Mock(side_effect=[True])), \
         patch.object(inspect.getmodule(SLICPandora), 'Exists', new=Mock(side_effect=IOError('dont call me'))):
      assertDiracSucceeds(self.slic._checkConsistency('myTestJob'), self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
:synopsis: RequestOperation to handle log file failover

Uploads log files to the LogSE that were uploaded to a GridSE as a failover
mechanism. Removes log files from the GridSE

Adapted from LHCbDirac.

"""

from __future__ import absolute_import
import os

from DIRAC import S_OK, S_ERROR
from DIRAC.FrameworkSystem.Client.MonitoringClient import gMonitor
from DIRAC.RequestManagementSystem.private.OperationHandlerBase import OperationHandlerBase

__RCSID__ = "$Id$"


class LogUpload(OperationHandlerBase):
  """LogUpload operation handler."""

  def __init__(self, operation=None, csPath=None):
    """c'tor.

    :param self: self reference
    :param Operation operation: Operation instance
    :param str csPath: CS path for this handler
    """
    # # base class ctor
    OperationHandlerBase.__init__(self, operation, csPath)
    # # gMonitor stuff
    gMonitor.registerActivity("LogUploadAtt", "Log upload attempted",
                               "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM)
    gMonitor.registerActivity("LogUploadOK", "Replications successful",
                               "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM)
    gMonitor.registerActivity("LogUploadFail", "Replications failed",
                               "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM)
    self.workDirectory = os.environ.get('LOGUPLOAD_CACHE', os.environ.get('AGENT_WORKDIRECTORY', '/tmp/LogUpload'))

  def __call__(self):
    """LogUpload operation processing."""
    # # list of targetSEs

    targetSEs = self.operation.targetSEList

    if len(targetSEs) != 1:
      self.log.error("wrong value for TargetSE list = %s, should contain only one target!" % targetSEs)
      self.operation.Error = "Wrong parameters: TargetSE should contain only one targetSE"
      for opFile in self.operation:

        opFile.Status = "Failed"
        opFile.Error = "Wrong parameters: TargetSE should contain only one targetSE"

        gMonitor.addMark("LogUploadAtt", 1)
        gMonitor.addMark("LogUploadFail", 1)

      return S_ERROR("TargetSE should contain only one target, got %s" % targetSEs)

    targetSE = targetSEs[0]
    targetWrite = self.rssSEStatus(targetSE, "WriteAccess")
    if not targetWrite["OK"]:
      self.log.error(targetWrite["Message"])
      for opFile in self.operation:
        opFile.Status = "Failed"
        opFile.Error = "Wrong parameters: %s" % targetWrite["Message"]
        gMonitor.addMark("LogUploadAtt", 1)
        gMonitor.addMark("LogUploadFail", 1)
      self.operation.Error = targetWrite["Message"]
      return S_OK()

    if not targetWrite["Value"]:
      self.operation.Error = "TargetSE %s is banned for writing"
      return S_ERROR(self.operation.Error)

    # # get waiting files
    waitingFiles = self.getWaitingFilesList()

    # # loop over files
    for opFile in waitingFiles:
      # # get LFN
      lfn = opFile.LFN
      self.log.info("processing file %s" % lfn)
      gMonitor.addMark("LogUploadAtt", 1)

      destinationFolder = '/'.join(lfn.split('/')[0:-1])
      destinationSubFolder = "%03d" % (int((os.path.basename(lfn)).split('_')[1].split('.')[0]) // 1000)
      destination = destinationFolder + '/' + destinationSubFolder

      logUpload = self.dm.replicate(lfn, targetSE, destPath=destination, localCache=self.workDirectory)
      if not logUpload["OK"]:
        gMonitor.addMark("LogUploadFail", 1)
#         self.dataLoggingClient().addFileRecord( lfn, "LogUploadFail", targetSE, "", "LogUpload" )
        self.log.error("completely failed to upload log file: %s" % logUpload["Message"])
        opFile.Error = str(logUpload["Message"])
        opFile.Attempt += 1
        self.operation.Error = opFile.Error
        if 'No such file or directory' in opFile.Error:
          opFile.Status = 'Failed'
        continue

      if lfn in logUpload['Value']:
        gMonitor.addMark("LogUploadOK", 1)
#         self.dataLoggingClient().addFileRecord( lfn, "LogUpload", targetSE, "", "LogUpload" )
        opFile.Status = 'Done'
        self.log.info("Uploaded %s to %s" % (lfn, targetSE))

    return S_OK()

  def setOperation(self, operation):  # pylint: disable=useless-super-delegation
    """operation and request setter.

    :param ~DIRAC.RequestManagementSystem.Client.Operation.Operation operation: operation instance
    :raises TypeError: if ``operation`` in not an instance of :class:`~DIRAC.RequestManagementSystem.Client.Operation.Operation`
    """
    super(LogUpload, self).setOperation(operation)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Create Conditional Overlay."""
from __future__ import print_function
from __future__ import absolute_import
from DIRAC.Core.Base import Script

Script.parseCommandLine()

from ILCDIRAC.Interfaces.API.NewInterface.Applications import OverlayInput, Marlin
from ILCDIRAC.Interfaces.API.NewInterface.UserJob import UserJob
from ILCDIRAC.Interfaces.API.DiracILC import DiracILC

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
fc = FileCatalogClient()

meta = {}
meta["ProdID"] = 651

res = fc.findFilesByMetadata(meta)
if not res['OK']:
  print("Found no files")
  exit()

lfns = res['Value']
print("found %s files" % len(lfns))


ovi = OverlayInput()
ovi.setEnergy(500.)
ovi.setBXOverlay(300)
ovi.setGGToHadInt(0.3)
ovi.setNbSigEvtsPerJob(10)
ovi.setBkgEvtType("gghad")
ovi.setDetectorModel("CLIC_ILD_CDR")

overlay = [True, False]

for ov in overlay:
  d = DiracILC(True, "repo_overlay_%s.rep" % ov)
  for lfn in lfns:
    j = UserJob()
    steeringf = "clic_ild_cdr_steering.xml"
    if ov:
      steeringf = "clic_ild_cdr_steering_overlay.xml"
      res = j.append(ovi)
      if not res['OK']:
        print(res['Message'])
        continue
    ma = Marlin()
    ma.setVersion("v0111Prod")
    ma.setGearFile("clic_ild_cdr.gear")
    ma.setSteeringFile(steeringf)
    ma.setInputFile("LFN:" + lfn)
    ma.setNbEvts(10)
    ma.setEnergy(500.)
    ma.setOutputRecFile("myrec_overlay_%s.slcio" % ov)
    ma.setOutputDstFile("mydst_overlay_%s.slcio" % ov)
    res = j.append(ma)
    if not res['OK']:
      print(res['Message'])
      exit()

    j.setCPUTime(86400)
    j.setOutputData("myrec_overlay_%s.slcio" % ov, "some/path")
    j.setName("SomeName")
    j.setJobGroup("SomeGroup")
    res = d.checkparams(j)
    if not res['OK']:
      print(res['Message'])
      exit()

    j.submit(d)
